@@ -174,6 +174,7 @@ class PulpPushItem(object):
             return f_return([])
 
         unit_type = items[0].unit_type
+        unit_fields = items[0].unit_fields
 
         if unit_type is None:
             # This means that the item doesn't map to a specific single unit type
@@ -182,10 +183,10 @@ class PulpPushItem(object):
             return f_return(items)
 
         crit = Criteria.and_(
-            Criteria.with_unit_type(unit_type),
+            Criteria.with_unit_type(unit_type, unit_fields=unit_fields),
             Criteria.or_(*[item.criteria() for item in items]),
         )
-        LOG.info("Doing Pulp search: %s", crit)
+        LOG.debug("Doing Pulp search: %s", crit)
 
         units_f = pulp_client.search_content(crit)
         matcher = partial(cls.match_items_units, items)
@@ -531,6 +532,19 @@ class PulpPushItem(object):
         """
         raise NotImplementedError()
 
+    @property
+    def unit_fields(self):
+        """Field name(s) required when searching for units. By default, all fields
+        will be queried from Pulp.
+
+        Subclasses MAY override this method to return a specific list of fields.
+
+        This may improve performance by avoiding the query of unnecessary fields,
+        but care must be taken to include the full list of required fields needed
+        in order to push content of this type.
+        """
+        return None
+
     @property
     def supports_signing(self):
         """True if this item supports signing.
